#!/usr/bin/python

""" CeibalSCAConf -- Ceibal SCA XO client, configuration file parser.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    May 2009.

    VERSION: $Id$

    TODO:
      * Define default values for options.
"""

import os, re

from ConfigParser import SafeConfigParser

from CeibalSCAErrors import *

# The configuration file
CEIBAL_CONF_FILE = "/etc/CeibalSCA.conf"


# Configuration file sections
CONF_SECTION_WISPR 	= "CeibalSCAWispr"
CONF_SECTION_NETWORK 	= "CeibalSCANetwork"
CONF_SECTION_HTTPS	= "CeibalSCAHttps"
CONF_SECTION_URL	= "CeibalSCAURL"
CONF_SECTION_SECURITY	= "CeibalSCASecurity"
CONF_SECTION_SLEEP 	= "CeibalSCASleep"
CONF_SECTION_LOG	= "CeibalSCALog"


# The default value for all options
DEFAULT_PUBLIC_URL	 	= "http://www.google.com.uy"
DEFAULT_PUBLIC_URL_CONTENT	= "<title>Google"
DEFAULT_NETWORK_INTERFACE	= "eth0"
DEFAULT_NETWORK_ESSID		= "Ceibal_XX"
DEFAULT_NETWORK_RANGE	 	= "10.20.1.0/24"
DEFAULT_CHECK_CA	 	= True
DEFAULT_CA_CERTIFICATE	 	= "/etc/sca/LATU-CA-cert.pem"
DEFAULT_CA_CERTIFICATE_DEPTH	= 1
DEFAULT_REGISTER_URL	 	= "https://172.16.40.4/pca/register"
DEFAULT_REAUTHENTICATION_URL	= "https://172.16.40.4/pca/reauth"
DEFAULT_SYNCHRONIZATION_URL	= "https://172.16.40.4/pca/sync"
DEFAULT_RESYNCHRONIZATION_URL	= "https://172.16.40.4/pca/resync"
DEFAULT_SCA_SECURITY_PATH	= "/etc/sca"
DEFAULT_OPENFIRMWARE_PATH	= "/ofw"
DEFAULT_REGISTER_SLEEP		= 10
DEFAULT_AUTH_SLEEP		= 5
DEFAULT_REAUTH_SLEEP		= 5
DEFAULT_SYNC_SLEEP		= 5
DEFAULT_RESYNC_SLEEP		= 10
DEFAULT_LOG_OUTPUT		= "stdout"
DEFAULT_DEBUG_LOG	 	= "/var/run/CeibalSCAd/debug_log"
DEFAULT_ERROR_LOG	 	= "/var/run/CeibalSCAd/error_log"


class CeibalSCAConf:
    """ Ceibal SCA XO client configuration file parser. This class is
        a SINGLETON, please don't call constructor directly. Use
        CeibalSCAConf.get_instance(), instead.

        Also, as we are a deamon, we might need to reread config file,
        so don't save options as attributes in non temporary
        instances, or they won't get updated.

        In case you need to compute something costly with an option
        (NM network range, for instance), this class provides a
        config_changed callback.
    """

    instance = None


    class Singleton:
        """ Auxiliary class """
        def __call__( self, *args, **kw ) :

            # If an instance of CeibalSCAConf does not exist, create one
            # and assign it to CeibalSCAConf.instance.

            if (CeibalSCAConf.instance is None):
                object = CeibalSCAConf()
                CeibalSCAConf.instance = object

            # Return CeibalSCAConf.instance, which should contain a
            # reference to the only instance of CeibalSCAConf in the
            # system.

            return CeibalSCAConf.instance

    get_instance = Singleton()

    def __init__(self):
        """ DO NOT CALL THIS METHOD! """
        if (not CeibalSCAConf.instance is None) :
            raise CeibalSCAException('Only one instance of CeibalSCAConf is allowed!')
        CeibalSCAConf.instance = self

        self._parser = SafeConfigParser()
        try:
            self._parser.readfp(open(CEIBAL_CONF_FILE))
        except:
            # It looks like there's no way of using CeibalSCALog
            # here... it would be interesting to be able to log this
            # error.
            pass

        self.callbacks = []

    def add_config_changed_callback(self, callback):
        """ Add a callback function. This function is called when
            configuration file is reread.

            Arguments:
              + Callback function (must receive no arguments)
        """
        self.callbacks.append(callback)

        
    def remove_config_changed_callback(self, callback):
        """ Remove a callback function.
        """
        try:
            self.callbacks.remove(callback)
        except:
            pass
        
    def reread_conf(self):
        """ Reread configuration file.
        """
        try:
            self._parser.readfp(open(CEIBAL_CONF_FILE))
        except:
            # It looks like there's no way of using CeibalSCALog
            # here... it would be interesting to be able to log this
            # error.
            self._parser = SafeConfigParser()
            
        for c in self.callbacks:
            c()

    def _get_option(self, section, opt_name, default=""):
        """ Returns the given option in the given section as a string.
            If option cannot be retrieved, use default.

            Arguments:
              + Section name
              + Option name
              + Default string value
        """
        try:
            return self._parser.get(section, opt_name)
        except:
            return default

    def _get_bool(self, section, bool_name, default=False):
        """ Returns the value of the option as a boolean. If option
            cannot be retrieved, use default.

            In config file, value can be "1", "yes", "on" or "0",
            "no", "off".

            Arguments:
              + Section name
              + Option name
              + Default boolean value
        """
        try:
            return self._parser.getboolean(section, bool_name)
        except:
            return default


    # Particular options getters
    def get_public_url(self):
        return self._get_option(CONF_SECTION_WISPR, 'public_url', DEFAULT_PUBLIC_URL)

    def get_public_url_content(self):
        return self._get_option(CONF_SECTION_WISPR, 'public_url_content', DEFAULT_PUBLIC_URL_CONTENT)

    def get_network_interface(self):
        return self._get_option(CONF_SECTION_NETWORK, 'network_interface', DEFAULT_NETWORK_INTERFACE)

    def get_network_essid(self):
        return self._get_option(CONF_SECTION_NETWORK, 'network_essid', DEFAULT_NETWORK_ESSID)

    def get_network_range(self):
        return self._get_option(CONF_SECTION_NETWORK, 'network_range', DEFAULT_NETWORK_RANGE)

    def get_network_proxy(self):
        proxy = {}
        if (self._parser.has_option(CONF_SECTION_NETWORK, 'network_http_proxy')):
            proxy['http'] = self._get_option(CONF_SECTION_NETWORK, 'network_http_proxy')

        if (self._parser.has_option(CONF_SECTION_NETWORK, 'network_https_proxy')):
            proxy['https'] = self._get_option(CONF_SECTION_NETWORK, 'network_https_proxy')

        if (self._parser.has_option(CONF_SECTION_NETWORK, 'network_ftp_proxy')):
            proxy['ftp'] = self._get_option(CONF_SECTION_NETWORK, 'network_ftp_proxy')

        if (proxy == {}):
            proxy = None
        return proxy

    def get_check_ca(self):
        return self._get_bool(CONF_SECTION_HTTPS, 'check_ca', DEFAULT_CHECK_CA)

    def get_ca_certificate(self):
        return self._get_option(CONF_SECTION_HTTPS, 'ca_certificate', DEFAULT_CA_CERTIFICATE)

    def get_ca_certificate_depth(self):
        try:
            return self._parser.getint(CONF_SECTION_HTTPS, 'ca_certificate_depth')
        except:
            return DEFAULT_CA_CERTIFICATE_DEPTH

    def get_register_url(self):
        return self._get_option(CONF_SECTION_URL, 'register_url', DEFAULT_REGISTER_URL)

    def get_reauthentication_url(self):
        return self._get_option(CONF_SECTION_URL, 'reauthentication_url', DEFAULT_REAUTHENTICATION_URL)

    def get_synchronization_url(self):
        return self._get_option(CONF_SECTION_URL, 'synchronization_url', DEFAULT_SYNCHRONIZATION_URL)

    def get_resynchronization_url(self):
        return self._get_option(CONF_SECTION_URL, 'resynchronization_url', DEFAULT_RESYNCHRONIZATION_URL)

    def get_sca_security_path(self):
        return self._get_option(CONF_SECTION_SECURITY, 'sca_security_path', DEFAULT_SCA_SECURITY_PATH)

    def get_openfirmware_path(self):
        return self._get_option(CONF_SECTION_SECURITY, 'openfirmware_path', DEFAULT_OPENFIRMWARE_PATH)


    def get_register_sleep(self):
        try:
            return self._parser.getint(CONF_SECTION_SLEEP, 'register_sleep')
        except:
            return DEFAULT_REGISTER_SLEEP

    def get_auth_sleep(self):
        try:
            return self._parser.getint(CONF_SECTION_SLEEP, 'auth_sleep')
        except:
            return DEFAULT_AUTH_SLEEP

    def get_reauth_sleep(self):
        try:
            return self._parser.getint(CONF_SECTION_SLEEP, 'reauth_sleep')
        except:
            return DEFAULT_REAUTH_SLEEP

    def get_sync_sleep(self):
        try:
            return self._parser.getint(CONF_SECTION_SLEEP, 'sync_sleep')
        except:
            return DEFAULT_SYNC_SLEEP

    def get_resync_sleep(self):
        try:
            return self._parser.getint(CONF_SECTION_SLEEP, 'resync_sleep')
        except:
            return DEFAULT_RESYNC_SLEEP

    def get_log_output(self):
        return self._get_option(CONF_SECTION_LOG, 'log_output', DEFAULT_LOG_OUTPUT)

    def get_debug_log(self):
        return self._get_option(CONF_SECTION_LOG, 'debug_log', DEFAULT_DEBUG_LOG)

    def get_error_log(self):
        return self._get_option(CONF_SECTION_LOG, 'error_log', DEFAULT_ERROR_LOG)

    def get_client_version(self):
	aux = os.popen("rpm -q CeibalSCA")
        version = aux.readline()
        v = re.search('^CeibalSCA-(\d+\.\d+-\d+).*$', version)
	aux.close()
        return v.group(1)

    def get_OS_version(self):
        aux = os.popen("uname -r")
        uname = aux.readline()
	if( os.path.exists("/boot/olpc_build") ):
        	aux2 = os.popen("cat /boot/olpc_build")
        	olpcBuild = aux2.readline()
		version = olpcBuild[0:len(olpcBuild)-1] + " - " + uname[0:len(uname)-1] 
		aux.close()
		aux2.close()
	else:
		version = uname[0:len(uname)-1]
		aux.close()	
        return version

