#!/usr/bin/python

""" CeibalSCALog -- Ceibal SCA library. Logging facility.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCALog.py 568 2009-07-12 12:49:53Z jcampo $
"""
import inspect
import sys
from syslog import *
import time

from CeibalSCAErrors import *
from CeibalSCAConf import *

class CeibalSCALog:
    """ Ceibal SCA XO client logging facility. This class is a
        SINGLETON, please don't call constructor directly. Use
        CeibalSCALog.get_instance(), instead.
    """

    instance = None
    
    class Singleton:
        """ Auxiliary class """
        def __call__( self, *args, **kw ) :

            # If an instance of CeibalSCALog does not exist, create one
            # and assign it to CeibalSCALog.instance.

            if (CeibalSCALog.instance is None):
                object = CeibalSCALog()
                CeibalSCALog.instance = object

            # Return CeibalSCALog.instance, which should contain a
            # reference to the only instance of CeibalSCALog in the
            # system.

            return CeibalSCALog.instance

    get_instance = Singleton()


    def __init__ (self):
        """ DO NOT CALL THIS METHOD! """
        self.ceibal_conf = CeibalSCAConf.get_instance()
        self.ceibal_conf.add_config_changed_callback(self._open_log_output)

        self.err_file = None
        self.debug_file = None

        self._open_log_output()


    def _open_log_output(self):
        if (self.err_file):
            self.err_file.close()
        if (self.debug_file):
            self.debug_file.close()

        output = self.ceibal_conf.get_log_output()
        err_file = self.ceibal_conf.get_error_log()
        debug_file = self.ceibal_conf.get_debug_log()
        
        if (output != "file" and output != "stdout" and output != "syslog"):
            raise CeibalSCAException ("Invalid log output")
        self.output = output

        self.err_file = None
        self.debug_file = None
        
        if (output == "file"):
            if ((not err_file) or (not debug_file)):
                raise CeibalSCAException ("Invalid log files")
            self.err_file = open (err_file, "a")
            self.debug_file = open (debug_file, "a")
        if (output == "syslog"):
            openlog("CeibalSCA")
            
    def __del__ (self):
        if (self.err_file):
            self.err_file.close()
        if (self.debug_file):
            self.debug_file.close()
        

    def error (self, message):

        """ Prints an error, either to a file, stderr or to syslog
            (LOG_ERR priority).

            Arguments:
              + The message to print
        """
        if (self.output == "file"):
            #self.err_file.write("%s - CeibalSCA: %s\n" % (time.asctime(), message))
            self.err_file.write("%s - CeibalSCA:[%s:%s] %s\n" % (time.asctime(),inspect.currentframe().f_back.f_code.co_filename, inspect.currentframe().f_back.f_lineno, message))
            self.err_file.flush()
        elif (self.output == "stdout"):
            #sys.stderr.write("%s - CeibalSCA: %s\n" % (time.asctime(), message))
            sys.stderr.write("%s - CeibalSCA:[%s:%s] %s\n" % (time.asctime(),inspect.currentframe().f_back.f_code.co_filename, inspect.currentframe().f_back.f_lineno, message))
            sys.stderr.flush()
        else:
            syslog (LOG_ERR, message)

    def debug (self, message):

        """ Prints a debug message, either to stdout, or to syslog
            (LOG_DEBUG priority).

            Arguments:
              + The message to print
        """

        if (self.output == "file"):
            # self.debug_file.write("%s - CeibalSCA: %s\n" % (time.asctime(), message))
            self.debug_file.write("%s - CeibalSCA:[%s:%s] %s\n" % (time.asctime(),inspect.currentframe().f_back.f_code.co_filename, inspect.currentframe().f_back.f_lineno, message))
            self.debug_file.flush()
        elif (self.output == "stdout"):
            #sys.stdout.write("%s - CeibalSCA: %s\n" % (time.asctime(), message))
            sys.stdout.write("%s - CeibalSCA:[%s:%s] %s\n" % (time.asctime(),inspect.currentframe().f_back.f_code.co_filename, inspect.currentframe().f_back.f_lineno, message))
            sys.stdout.flush()
        else:
            syslog (LOG_DEBUG, message)

        
    
               
