#!/usr/bin/python

""" CeibalSCAProtocol -- Ceibal SCA XO client, protocol functions
    implementation.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id$
"""

from binascii import hexlify

from CeibalSCASeed import *
from CeibalSCAUtils import *
from CeibalSCAHttps import *
from CeibalSCAErrors import *
from CeibalSCALog import *
from CeibalSCAXML import *
from CeibalSCAConf import *
from CeibalSCAWispr import *

class CeibalSCAProtocol:
    """ Functions for each protocol phase.
    """

    def __init__(self):
        """ Create CeibalSCAProtocol instance.
        """
        self.log = CeibalSCALog.get_instance()
        self.ceibal_http = CeibalHTTP()
        self.ceibal_conf = CeibalSCAConf.get_instance()
        self.wispr = CeibalSCAWispr(self.ceibal_http)

    def register(self):
        """ Register the XO to the PCA. Raise exception if FirstSeed
            already exists. No return value.

            Arguments: None
        """
        xml_parser = CeibalSCAXML()

        first_seed = FirstSeed()
        first_seed = first_seed.load()

        if (first_seed):
            raise CeibalSCASeedExists()
        first_seed = FirstSeed()

        sn = read_sn()
        xo_uuid = read_uuid()
        mac = get_mac_address()
        first_seed.setValue(create_nonce())
        cliVersion = self.ceibal_conf.get_client_version()
        osVersion = self.ceibal_conf.get_OS_version()

        data = {'sn': sn, 'uuid': xo_uuid, 'mac': mac, 'firstSeed': first_seed.getValue(), 'cliVersion': cliVersion, 'osVersion': osVersion}

        self.log.debug("Registering with SN=\"%s\", UUID=\"%s\"" % (sn, xo_uuid))

        check_https_url(self.ceibal_conf.get_register_url())
        res = self.ceibal_http.post_data (self.ceibal_conf.get_register_url(), data)
        contents = res.read()

        xml_contents = xml_parser.read_embedded_xml (contents, "response")

        error_code = int (xml_parser.get_unique_text_value(xml_contents, 'errorCode'))
        times_used = xml_parser.get_unique_text_value(xml_contents, 'timesUsed', True)
        refresh_interval = xml_parser.get_unique_text_value(xml_contents, 'refreshInterval', True)

        max_login_fail = xml_parser.get_unique_text_value(xml_contents, 'maxLoginFail', True)
        max_resync_fail = xml_parser.get_unique_text_value(xml_contents, 'maxResyncFail', True)
        max_reauth_fail = xml_parser.get_unique_text_value(xml_contents, 'maxReauthFail', True)
        max_sync_fail = xml_parser.get_unique_text_value(xml_contents, 'maxSyncFail', True)

        session_timeout = xml_parser.get_unique_text_value(xml_contents, 'sessionTime', True)

        new_token = xml_parser.get_unique_text_value(xml_contents, 'nonce', True)
        if (error_code != 0):
            raise CeibalSCAException("PCA returned error code %d" % error_code)
        if (not new_token):
            raise CeibalSCAException("PCA did not send nonce in registration")
        first_seed.setStateSynchronized()
        first_seed.save()

        current_seed = CurrentSeed()
        current_seed.setValue(onewayfunction(sn + xo_uuid + first_seed.getValue() +
                                             first_seed.getValue() + new_token))
        current_seed.setStateSynchronized()
        current_seed.setLastUsed(getCurrentTime())

        if (times_used):
          current_seed.setTimesUsed(int(times_used))
        if (refresh_interval):
          current_seed.setInterval(int(refresh_interval))
        if (max_login_fail):
            current_seed.setMaxLoginFail(int(max_login_fail))
        if (max_resync_fail):
            current_seed.setMaxResyncFail(int(max_resync_fail))
        if (max_reauth_fail):
            current_seed.setMaxReauthFail(int(max_reauth_fail))
        if (max_sync_fail):
            current_seed.setMaxSyncFail(int(max_sync_fail))
        if (session_timeout):
            current_seed.setSessionTimeout(int(session_timeout))

        current_seed.save()

    def authenticate(self):
        current_seed = CurrentSeed()
        current_seed = current_seed.load()
        if (not current_seed):
            raise CeibalSCAProtocolException(
                "Could not find current seed when authenticating")

        user = onewayfunction(read_sn() + read_uuid()) + "@xo.ceibal.edu.uy"
        password = current_seed.getValue()

        self.wispr.authenticate(user, password)

        current_seed.incrCount()
        current_seed.setLastUsed(getCurrentTime())
        current_seed.resetReauthTimestamp()
        current_seed.save()

    def logoff(self):
        self.wispr.logoff()

    def reauthenticate(self):
        xml_parser = CeibalSCAXML()
        first_seed = FirstSeed()
        first_seed = first_seed.load()
        current_seed = CurrentSeed()
        current_seed = current_seed.load()
        if (not current_seed):
            raise CeibalSCAProtocolException(
                "Could not find current seed when reauthenticating")

        user = onewayfunction(read_sn() + read_uuid()) + "@xo.ceibal.edu.uy"

        ####fs_hash = long(onewayfunction(first_seed.getValue()), 16)
        ####secret = long2hex(fs_hash ^ long(current_seed.getValue(), 16))
	secret = self.__encrypt(current_seed.getValue(), first_seed.getValue())
        current_seed.incrReauthTimestamp()
        #timestamp = long2hex(fs_hash ^ current_seed.getReauthTimestamp())

	cliVersion = self.ceibal_conf.get_client_version()

        data = {'radiusUsername': user, 'hashRadiusPassword': secret, 'cliVersion': cliVersion }#, 'timestamp': timestamp}
        self.log.debug("Re-authenticating...")

        check_https_url(self.ceibal_conf.get_reauthentication_url())
        res = self.ceibal_http.post_data (self.ceibal_conf.get_reauthentication_url(), data)
        contents = res.read()

        # jdcampo - Added to debug
        self.log.debug(contents)

        xml_contents = xml_parser.read_embedded_xml (contents, "response")

        error_code = int (xml_parser.get_unique_text_value(xml_contents, 'errorCode'))

        if (error_code != 0):
            raise CeibalSCAException("PCA returned error code %d" % error_code)

        current_seed.setLastUsed(getCurrentTime())

        current_seed.save()

    def synchronize(self):
        xml_parser = CeibalSCAXML()

        first_seed = FirstSeed()
        first_seed = first_seed.load()
        if (not first_seed):
            raise CeibalSCAException("Could not find first seed when synchronizing")

        current_seed = CurrentSeed()
        current_seed = current_seed.load()
        if (not current_seed):
            raise CeibalSCAException("Could not find current seed when synchronizing")

        sn = read_sn()
        xo_uuid = read_uuid()

        #####fs_hash = onewayfunction(first_seed.getValue())
        #####secret = long2hex(long(fs_hash, 16) ^ long(current_seed.getValue(), 16))
	secret = self.__encrypt(current_seed.getValue(), first_seed.getValue())

	cliVersion = self.ceibal_conf.get_client_version()

        data = {'sn': sn, 'uuid': xo_uuid, 'hashCurrentSeed': secret, 'cliVersion': cliVersion}

        self.log.debug("Synchronizing with SN=\"%s\", UUID=\"%s\"" % (sn, xo_uuid))

        check_https_url(self.ceibal_conf.get_synchronization_url())
        res = self.ceibal_http.post_data (self.ceibal_conf.get_synchronization_url(), data)
        contents = res.read()

        xml_contents = xml_parser.read_embedded_xml (contents, "response")

        error_code = int (xml_parser.get_unique_text_value(xml_contents, 'errorCode'))
        times_used = xml_parser.get_unique_text_value(xml_contents, 'timesUsed', True)
        refresh_interval = xml_parser.get_unique_text_value(xml_contents, 'refreshInterval', True)
        if (times_used):
          times_used = int(times_used)
        if (refresh_interval):
          refresh_interval = int(refresh_interval)

        new_token = xml_parser.get_unique_text_value(xml_contents, 'nonce', True)

        if (error_code != 0):
            raise CeibalSCAException("PCA returned error code %d" % error_code)

        current_seed.setValue(onewayfunction(sn + xo_uuid + current_seed.getValue() +
                                             first_seed.getValue() + new_token))

        current_seed.setStateSynchronized()
        current_seed.setTimesUsed(times_used)
        current_seed.resetCount()
        current_seed.setInterval(refresh_interval)
        current_seed.setLastUsed(getCurrentTime())

        current_seed.save()

    def resynchronize(self):
        xml_parser = CeibalSCAXML()

        first_seed = FirstSeed()
        first_seed = first_seed.load()
        if (not first_seed):
            raise CeibalSCAException("Could not find first seed when resynchronizing")

        current_seed = CurrentSeed()
        current_seed = current_seed.load()
        if (not current_seed):
            raise CeibalSCAException("Could not find current seed when resynchronizing")

        sn = read_sn()
        xo_uuid = read_uuid()

        ####fs_hash = onewayfunction(first_seed.getValue())
        ####secret = long2hex(long(fs_hash, 16) ^ long(current_seed.getValue(), 16))
	secret = self.__encrypt(current_seed.getValue(), first_seed.getValue())

        md = onewayfunction(sn + xo_uuid + first_seed.getValue())

	cliVersion = self.ceibal_conf.get_client_version()

        data = {'sn': sn, 'uuid': xo_uuid, 'hashFirstSeed': md, 'hashCurrentSeed': secret, 'cliVersion': cliVersion}

        self.log.debug("Resynchronizing...")

        check_https_url(self.ceibal_conf.get_resynchronization_url())
        res = self.ceibal_http.post_data (self.ceibal_conf.get_resynchronization_url(), data)
        contents = res.read()

        xml_contents = xml_parser.read_embedded_xml (contents, "response")

        error_code = int (xml_parser.get_unique_text_value(xml_contents, 'errorCode'))
        times_used = xml_parser.get_unique_text_value(xml_contents, 'timesUsed', True)
        refresh_interval = xml_parser.get_unique_text_value(xml_contents, 'refreshInterval', True)
        max_login_fail = xml_parser.get_unique_text_value(xml_contents, 'maxLoginFail', True)
        max_resync_fail = xml_parser.get_unique_text_value(xml_contents, 'maxResyncFail', True)
        max_reauth_fail = xml_parser.get_unique_text_value(xml_contents, 'maxReauthFail', True)
        max_sync_fail = xml_parser.get_unique_text_value(xml_contents, 'maxSyncFail', True)
        session_timeout = xml_parser.get_unique_text_value(xml_contents, 'sessionTime', True)

        new_token = xml_parser.get_unique_text_value(xml_contents, 'nonce', True)
        if (error_code != 0):
            raise CeibalSCAException("PCA returned error code %d" % error_code)
        if (not new_token):
            raise CeibalSCAException("PCA did not send nonce in resynchronization")

        current_seed.setValue(onewayfunction(sn + xo_uuid + current_seed.getValue() +
                                             first_seed.getValue() + new_token))
        current_seed.setStateSynchronized()
        current_seed.setLastUsed(getCurrentTime())
        current_seed.resetCount()

        if (times_used):
          current_seed.setTimesUsed(int(times_used))
        if (refresh_interval):
          current_seed.setInterval(int(refresh_interval))
        if (max_login_fail):
            current_seed.setMaxLoginFail(int(max_login_fail))
        if (max_resync_fail):
            current_seed.setMaxResyncFail(int(max_resync_fail))
        if (max_reauth_fail):
            current_seed.setMaxReauthFail(int(max_reauth_fail))
        if (max_sync_fail):
            current_seed.setMaxSyncFail(int(max_sync_fail))
        if (session_timeout):
            current_seed.setSessionTimeout(int(session_timeout))

        current_seed.save()

    def check_public_url(self):
        return self.wispr.check_public_url()

    def __encrypt(self, data, key):
        key_hash = onewayfunction(key)
        n = len(key_hash)
        secret = '';
        for i in range(n):
                a =  key_hash[i];
                b = data[i];
                secret = secret + hex_digit(long(a, 16) ^ long(b, 16))
        return secret
