#!/usr/bin/python

""" CeibalSCAState -- Ceibal SCA XO client state class.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    May 2009.

    VERSION: $Id$
"""

import os

from CeibalSCALog import *
from CeibalSCAConf import *
from CeibalSCAErrors import *
from CeibalSCAUtils import *

# Possible client states:
CS_UNKNOWN              = 0     # Unknown state
CS_INACT_NOTREG         = 1     # Inactive, Not registered
CS_ACT_NOTREG           = 2     # Active, Not registered
CS_ACT_REGNG            = 3     # Active, Registering
CS_ACT_SYNC             = 4     # Active, Synchronized
CS_INACT_SYNC           = 5     # Inactive, Synchronized
CS_AUTHNG_SYNC          = 6     # Authenticating, Synchronized
CS_AUTH_SYNC            = 7     # Authenticated, Synchronized
CS_AUTH_SYNCNG          = 8     # Authenticated, Synchronizing
CS_REAUTHNG_SYNC        = 9     # Reauthenticating, Synchronized
CS_ACT_DESYNC           = 10    # Active, Desynchronized
CS_ACT_RESYNCNG         = 11    # Active, Resynchronizing
CS_INACT_DESYNC         = 12    # Inactive, Desynchronized

# Client states string representation
_CS_STRING = ["UNKNOWN",
              "INACT_NOTREG",
              "ACT_NOTREG",
              "ACT_REGNG",
              "ACT_SYNC",
              "INACT_SYNC",
              "AUTHNG_SYNC",
              "AUTH_SYNC",
              "AUTH_SYNCNG",
              "REAUTHNG_SYNC",
              "ACT_DESYNC",
              "ACT_RESYNCNG",
              "INACT_DESYNC"]

_CLIENT_STATE_FILE = "client-state"

class CeibalSCAState:

    def __init__(self):
        self.ceibal_conf = CeibalSCAConf.get_instance()
        self.log = CeibalSCALog.get_instance()
        self._read_state()

    def set_state(self, transition, state):
        """ Set the client state.

            Arguments:
              + Transition string to print in debug info.
              + New client state
        """
        prev_state = str(self)
        self.state = int(state)
        self._write_state()
        self.log.debug("%s --%s--> %s" % (prev_state, transition, str(self)))

    def equal(self, state):
        return self.state == int(state)


    def _read_state(self):
        """ Read client state from file _CLIENT_STATE_FILE in
            security_path.
        """
        sec_path = self.ceibal_conf.get_sca_security_path()
        real_path = os.path.join(sec_path, _CLIENT_STATE_FILE)

        try:
            self.state = int(read_fs(real_path))
        except:
            self.state = CS_INACT_NOTREG
            self._write_state()

    def _write_state(self):
        """ Write client state to file _CLIENT_STATE_FILE in
            security_path.
        """
        sec_path = self.ceibal_conf.get_sca_security_path()
        real_path = os.path.join(sec_path, _CLIENT_STATE_FILE)

        try:
            write_fs(real_path, str(self.state))
        except Exception, e:
            raise CeibalSCAException("Cannot save client state: %s" % str(e))

    def __str__(self):
        if ((self.state < 0) or (self.state >= len(_CS_STRING))):
            return _CS_STRING[CS_UNKNOWN]
        return _CS_STRING[self.state]


# MAIN PROGRAM
# ============
if __name__ == '__main__':
    log = CeibalSCALog.get_instance()
    try:
        state = CeibalSCAState()
        state.set_state("Init", CS_INACT_NOTREG)
        state.set_state("Wlan UP", CS_ACT_NOTREG)
        state.set_state("Register", CS_ACT_REGNG)
        state.set_state("Registration OK", CS_ACT_SYNC)
        state.set_state("Authenticate", CS_AUTHNG_SYNC)
        state.set_state("Authentication OK", CS_AUTH_SYNC)
        state.set_state("End", CS_INACT_NOTREG)
    except Exception, e:
        log.error(str(e))
