#!/usr/bin/python

""" CeibalSCAUtils -- Ceibal SCA XO client, miscellaneous functions.

    Written by Mario Del Riego <mriego@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    March 2009.

    VERSION: $Id: CeibalSCAUtils.py 184 2009-05-26 03:07:38Z jcampo $
"""


import os # Random, path.join
from binascii import hexlify # To hex
import time # Actual time
import hashlib # SHA-1
import re

from CeibalSCAConf import *

CEIBAL_DATA_DISK = 1
CEIBAL_DATA_CMD  = 2



def getCurrentTime():
    """ Return the time as a floating point number expressed in
        seconds since the epoch, in UTC.
    """
    return time.time()

def write_fs(real_path,textdata):
    """ Write a file to File System.

        Arguments:
	  + The absolute file path.
    """
    f = open(real_path,'w');
    f.write(textdata)
    f.close()

def read_fs(real_path):
    """ Read a file from File System.

        Arguments:
	  + The absolute file path.
    """
    f = open(real_path,'r')
    res = f.read().rstrip("\n\0")
    f.close()
    return res

def read_ofw(dev_path):
    """ Read a node in the OFW device tree.

	Arguments:
	  + The path below OFW tree to read.
    """

    firmware_path = CeibalSCAConf.get_instance().get_openfirmware_path()
    real_path = os.path.join(firmware_path,dev_path)
    return read_fs(real_path)

def read_uuid():
    """ Return XO's UUID.

        Arguments: None
    """
    sourceData = get_source_data()
    if(sourceData == CEIBAL_DATA_CMD):
	return os.popen("olpc-hwinfo uuid").read(36)
    else:
    	return read_ofw(os.path.join('mfg-data','U#')).upper()

def read_sn():
    """ Return XO Serial Number.

        Arguments: None
    """
    sourceData = get_source_data()
    if(sourceData == CEIBAL_DATA_CMD):
	return os.popen("olpc-hwinfo serialnum").read(11)
    else:
	return read_ofw(os.path.join('mfg-data','SN')).upper()

def create_nonce():
    """ Return a string of 20 random bytes, suitable for cryptographic
        use.
    """
    return hexlify(os.urandom(20))

def onewayfunction(plaintext):
    """ Return the SHA1 hash of some data.

        Arguments:
	  + The data to hash
    """
	
    return hashlib.sha1(plaintext).hexdigest()

def get_mac_address():
    """ Return the MAC address of configured interface (see
        CeibalSCA.conf). Return None if the interface is not found.
    """
    if_name = CeibalSCAConf.get_instance().get_network_interface()
    mac = None
    for line in os.popen("/sbin/ifconfig %s" % if_name):
	if line.find('HWaddr') > -1:
	    mac = line.split()[4]
	    break
    return mac 

def hex_digit(num):
    if (num) < 10:
        return chr(num + ord('0'))
    else:
        return chr (num - 10 + ord('a'))
            
    
def long2hex(num):
    result = ""

    while (num != 0):
        rem = num % 16
        result = hex_digit(rem) + result
        num = num / 16

    return result
        

def check_https_url(url):
    """ Returns True if url is https, raises a CeibalSCAHTTPException
    	otherwise.

        Arguments:
          + The url to check.
    """
    if (re.search("^https://", url)):
        return True
    else:
        raise CeibalSCAHTTPException("URL \"%s\" is not secure" % url)
    
    
def get_source_data():
    """ Retunr a source for system information data.

	Arguments: None
    """
    snPath = os.path.join('mfg-data','SN')
    uuidPath = os.path.join('mfg-data','U#')
    firmware_path = CeibalSCAConf.get_instance().get_openfirmware_path()
    existSN = os.path.exists(os.path.join(firmware_path,snPath))
    existUUID = os.path.exists(os.path.join(firmware_path,uuidPath))

    if( existSN and existUUID  ):
	return CEIBAL_DATA_DISK 
    else:
	return CEIBAL_DATA_CMD




