#!/usr/bin/python

""" wispr -- WISPr client implementation library.

    This client expects the XML responses from the server embedded in
    HTML comments.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCAWispr.py 339 2009-06-03 17:57:23Z jcampo $

    TODO:
      + Proxy Reply functionality is missing in this version.
"""

import urllib2, re, os, time

from CeibalSCAErrors import *
from CeibalSCALog import *
from CeibalSCAHttps import *
from CeibalSCAXML import *
from CeibalSCAConf import *
from CeibalSCAUtils import *
from CeibalSCAUrl import *

# Wispr Message Types
REDIRECT_MT	= 100
PROXY_MT	= 110
AUTH_MT		= 120
LOGOFF_MT	= 130
POLL_REPLY_MT	= 140
ABORT_REPLY_MT	= 150

str_message_types = {str(REDIRECT_MT): "Initial Redirect Message",
                     str(PROXY_MT): "Proxy Notification",
                     str(AUTH_MT): "Authentication Notification",
                     str(LOGOFF_MT): "Logoff Notification",
                     str(POLL_REPLY_MT): "Response to Auth Poll",
                     str(ABORT_REPLY_MT): "Response to Abort Login"
                     }

# Wispr Response Codes
OK_RC		= 0
LOGIN_OK_RC	= 50
LOGIN_FAIL_RC	= 100
RADIUS_ERR_RC	= 102
NO_RADIUS_RC	= 105
LOGOFF_OK_RC	= 150
LOGIN_ABORT_RC	= 151
PROXY_REPEAT_RC = 200
AUTH_PENDING_RC = 201
INTERNAL_ERR_RC = 255

str_response_codes = {str(OK_RC): "No Error",
                      str(LOGIN_OK_RC): "Login Succeeded",
                      str(LOGIN_FAIL_RC): "Login Failed",
                      str(RADIUS_ERR_RC): "RADIUS Server error/timeout",
                      str(NO_RADIUS_RC): "Server does not have RADIUS enabled",
                      str(LOGOFF_OK_RC): "Logoff Succeeded",
                      str(LOGIN_ABORT_RC): "Login Aborted",
                      str(PROXY_REPEAT_RC): "Proxy detection/Repeat operation",
                      str(AUTH_PENDING_RC): "Authentication Pending",
                      str(INTERNAL_ERR_RC): "Access Gateway Internal Error"
                      }


class CeibalSCAWispr:
    def __init__(self, ceibal_http):

        """ Create an instance of the Wispr class.

            Arguments:
              + CeibalHTTP instance to send HTTP requests.
        """
        self.log = CeibalSCALog.get_instance()
        self.ceibal_http = ceibal_http
        self.xml_parser = CeibalSCAXML()
        self.ceibal_conf = CeibalSCAConf.get_instance()
        self.url_handler = URL()
        
        self.ceibal_http.setFollowRedirect(False)

        self.access_procedure = None
        self.access_location = None
        self.location_name = None
        self.login_url = None
        self.abort_url = None
        self.message_type = None
        self.message_str = None
        self.response_code = None
        self.response_code_str = None
        self.reply_message = None
        self.login_results_url = None
        self.delay = None

    def authenticate(self, user, password):
        """ Authenticates the user to the WISPr server, by connecting
            to a public URL, and expecting a redirect to a login page.
            No return value.

            Raises Exception if login is not successful for any
            reason. Chacks that connection is correct by trying to
            access a known public URL.

            Arguments:
              + Username
              + Password
        """
        self.user = user
        self.password = password

        self.log.debug ("Connecting to %s..." % self.ceibal_conf.get_public_url())
        
        res = self.ceibal_http.open(self.ceibal_conf.get_public_url())
        contents = res.read()

        if (re.search(self.ceibal_conf.get_public_url_content(), contents)):
            raise WisprAlreadyLoggedinException()

        self.log.debug ("Redirected to login page. Parsing XML reply...")
        self._get_login_info(contents)
        
        self.log.debug ("POSTing credentials to \"%s\"..." % self.location_name)
        self._post_credentials()

        # Test if we are connected
        res = self.ceibal_http.open (self.ceibal_conf.get_public_url())

        contents = res.read()
        if (re.search(self.ceibal_conf.get_public_url_content(), contents)):
            self.log.debug ("Connected successfully!")
        else:
            raise WisprProtocolException("Could not reach public URL")

    def logoff(self):
        """ Logs off the user from the WISPr server, if we were
            already authenticated. No return value.

            Raises Exception if logoff is not successful for any
            reason.

            Arguments: None
        """
        self.log.debug ("Logging off...")
        try:
            logoff_url = self.url_handler.logoff
        except:
            raise WisprProtocolException("Don't know logoff URL")

        res = self.ceibal_http.open (logoff_url)
        
        contents = res.read()
        wispr_tag = self.xml_parser.read_embedded_xml(contents, "WISPAccessGatewayParam")

        self.message_type = int(self.xml_parser.get_unique_text_value(wispr_tag, "MessageType"))
        self.response_code = int(self.xml_parser.get_unique_text_value(wispr_tag, "ResponseCode"))

        self.message_str = self._get_str_message_type(self.message_type)
        self.response_code_str = self._get_str_response_code(self.response_code)

        if (self.message_type != LOGOFF_MT):
            raise WisprProtocolException ("Invalid message type.\n\
  Server replied \"%d - %s\"\n  Expected \"%d - %s\"" %
                                      (self.message_type, self.message_str,
                                       LOGOFF_MT, self._get_str_message_type(LOGOFF_MT)))

        if (self.response_code == LOGOFF_OK_RC):
            # Logoff Successful
            self.log.debug (self._get_str_response_code(self.response_code))
        else:
            raise WisprProtocolException(self._get_str_response_code(self.response_code),
                                         self.response_code)


    def check_public_url(self):
        try:
            res = self.ceibal_http.open(self.ceibal_conf.get_public_url())
            contents = res.read()
            if (re.search(self.ceibal_conf.get_public_url_content(), contents) is None):
                return False
            else:
                return True
        except:
            return False


    # PRIVATE FUNCTIONS
    # =================
    def _get_str_message_type (self, mt):
        try:
            return str_message_types[str(mt)]
        except:
            return "Unkonwn message type"

    def _get_str_response_code (self, rc):
        try:
            return str_response_codes[str(rc)]
        except:
            return "Unkonwn response code"

    def _get_login_info(self, contents):

        """ Sets instance attributes corresponding to a WISPr Redirect
            Reply. No return value.

            Extracts the embedded XML from a given HTML content, and
            sets attributes "access_procedure", "access_location",
            "location_name", "login_url", "abort_url", "message_type"
            and "response_code".

            Raises an exception if there is no embedded XML, the XML
            is not valid or any of the tag is not present. Raises
            WisprProtocolException if the message type is not a
            Redirect or the response code is not OK.

            Arguments:
              + The contents of the HTML page.
        """
        wispr_tag = self.xml_parser.read_embedded_xml(contents, "WISPAccessGatewayParam")

        # TODO - handle proxy reply
        self.access_procedure = self.xml_parser.get_unique_text_value(wispr_tag, "AccessProcedure")
        self.access_location = self.xml_parser.get_unique_text_value(wispr_tag, "AccessLocation")

        self.location_name = self.xml_parser.get_unique_text_value(wispr_tag, "LocationName")
        self.login_url = self.xml_parser.get_unique_text_value(wispr_tag, "LoginURL")
        self.abort_url = self.xml_parser.get_unique_text_value(wispr_tag, "AbortLoginURL")
        self.message_type = int(self.xml_parser.get_unique_text_value(wispr_tag, "MessageType"))
        self.response_code = int(self.xml_parser.get_unique_text_value(wispr_tag, "ResponseCode"))

        self.location_name = self.location_name.lstrip(" \n\r\m")
        self.location_name = self.location_name.rstrip(" \n\r\m")
        self.message_str = self._get_str_message_type(self.message_type)
        self.response_code_str = self._get_str_response_code(self.response_code)

        if (self.message_type != REDIRECT_MT):
            raise WisprProtocolException ("Invalid message type.\n\
  Server replied \"%d - %s\"\n  Expected \"%d - %s\"" %
                                      (self.message_type, self.message_str,
                                       REDIRECT_MT, self._get_str_message_type(REDIRECT_MT)))

        if (self.response_code != OK_RC):
            raise WisprProtocolException (self.response_code_str, self.response_code)

    def _get_auth_reply(self, contents):

        """ Sets instance attributes corresponding to a WISPr Auth
            Reply. No return value.

            Extracts the embedded XML from a given HTML content, and
            sets attributes "message_type", "response_code",
            "reply_message", "login_results_url" and "logoff_url".

            Raises an exception if there is no embedded XML, the XML
            is not valid, no message_type or response_code tag is
            present, the message type is not an Auth Reply or the
            login fails.

            Arguments:
              + The contents of the HTML page.
        """
        pending = True
        while (pending):
            wispr_tag = self.xml_parser.read_embedded_xml(contents, "WISPAccessGatewayParam")

            self.message_type = int(self.xml_parser.get_unique_text_value(wispr_tag, "MessageType"))
            self.response_code = int(self.xml_parser.get_unique_text_value(wispr_tag, "ResponseCode"))
            self.reply_message = self.xml_parser.get_unique_text_value(wispr_tag, "ReplyMessage", True)
            self.login_results_url = self.xml_parser.get_unique_text_value(wispr_tag, "LoginResultsURL", True)
            self.delay = self.xml_parser.get_unique_text_value(wispr_tag, "Delay", True)
            logoff_url = self.xml_parser.get_unique_text_value(wispr_tag, "LogoffURL", True)

            self.message_str = self._get_str_message_type(self.message_type)
            self.response_code_str = self._get_str_response_code(self.response_code)
            if (not self.delay is None):
                self.delay = int (self.delay)

            if (self.message_type != AUTH_MT and self.message_type != POLL_REPLY_MT):
                raise WisprProtocolException ("Invalid message type.\n\
  Server replied \"%d - %s\"\n  Expected Auth reply or Poll reply" %
                                              (self.message_type, self.message_str))
            

            if (self.response_code == LOGIN_OK_RC):
                # Login Successful
                if (logoff_url is None):
                    raise WisprProtocolException ("Login OK, but no LogoffURL received")
                self.url_handler.logoff = logoff_url
                
                if (self.reply_message):
                    self.log.debug (self.reply_message)
                else:
                    self.log.debug (self._get_str_response_code(self.response_code))
                pending = False

            elif (self.response_code == LOGIN_FAIL_RC):
                if (self.reply_message):
                    raise WisprLoginFailedException(self.reply_message)
                else:
                    raise WisprLoginFailedException(self._get_str_response_code(self.response_code))

            elif (self.response_code == AUTH_PENDING_RC):
                if (self.login_results_url is None):
                    raise WisprProtocolException ("Auth pending, but no LoginResultsURL received")

                if (self.delay and self.delay > 0):
                    self.log.debug ("Auth pending. Sleeping for %d seconds..." % self.delay)
                    time.sleep(self.delay)

                self.log.debug ("Polling Login Results...")

                res = self.ceibal_http.open (self.login_results_url)
                contents = res.read()

            else:
                if (self.reply_message):
                    raise WisprProtocolException (self.reply_message, self.response_code)
                else:
                    raise WisprProtocolException(self._get_str_response_code(self.response_code),
                                                 self.response_code)

    def _post_credentials(self):
        """ POSTs the user credentials to the login_url, if it is
            known. No return value.

            Also parses the server reply.

            Raises Exception if no login_url is yet known, there is a
            connection problem to the server, or the reply parsing
            fails.

            Arguments: None
        """

        if (self.login_url is None):
            raise WisprProtocolException("Don't know login URL")

        check_https_url(self.login_url)

        post_data = {'button': 'Login', 'UserName': self.user,
                     'Password': self.password, 'FNAME': "0",
                     'OriginatingServer': self.ceibal_conf.get_public_url()}

        res = self.ceibal_http.post_data(self.login_url, post_data)
        contents = res.read()
        
        try:
            if (not re.search(self.ceibal_conf.get_public_url_content(), contents)):
                self._get_auth_reply(contents)
        except WisprLoginFailedException, e:
            # Login failed, don't abort login
            raise e
        except Exception, e:
            self._abort_login()
            raise e


    def _abort_login(self):
        """ Aborts a failed login. Only prints debug information if
            abort is not successful.

            Arguments: None.
        """
        if (not self.abort_url):
            self.log.debug("No abort URL known when trying to abort login.")
            return

        self.log.debug("Aborting login...")
        res = self.ceibal_http.open (self.abort_url)
        contents = res.read()

        wispr_tag = self.xml_parser.read_embedded_xml(contents, "WISPAccessGatewayParam")
        
        self.message_type = int(self.xml_parser.get_unique_text_value(wispr_tag, "MessageType"))
        self.response_code = int(self.xml_parser.get_unique_text_value(wispr_tag, "ResponseCode"))
        logoff_url = self.xml_parser.get_unique_text_value(wispr_tag, "LogoffURL", True)
        if (logoff_url):
            self.url_handler.logoff = logoff_url

        self.message_str = self._get_str_message_type(self.message_type)
        self.response_code_str = self._get_str_response_code(self.response_code)

        if (self.message_type != ABORT_REPLY_MT):
            self.log.debug ("Invalid message type.\n\
  Server replied \"%d - %s\"\n  Expected Abort reply" %
                            (self.message_type, self.message_str))
            
        if (self.response_code == LOGIN_OK_RC):
            self.log.debug("Warning, login OK in abort reply.")
        elif (self.response_code == LOGIN_ABORT_RC):
            self.log.debug("Login aborted")
        else:
            self.log.debug(self._get_str_response_code(self.response_code))
