#!/usr/bin/python

""" CeibalSCA_NM -- Handles NetworkManager communication for the
    Ceibal SCA XO client. This is the base class, subclass for a
    particular NM version.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCA_NM.py 542 2009-07-05 12:41:50Z jcampo $


    To create a new derived class for another version of NM, use
    CeibalSCA_NM_0_6_5 as an example.
    You have to implement __init__ method, with the following in mind:
      * Call this base class constructor, like this:
          CeibalSCA_NM.__init__(self)
      * Configure NM to tell you when its state changes. This is
        version dependant. Read the NetworkManager DBUS API for ideas.

    In the callback from NM, do:
      * Call _nm_connected method when starting a new connection.
      * Call _nm_disconnected method when disconnecting.

    Implement a new _get_nm_connection_params, following the behaviour
    defined in that function documentation.

    Remember to add your new version handler in CeibalSCA_NM_Factory.
"""

import re, sys, platform, time
from struct import pack, unpack
from string import join, atol

from CeibalSCALog import *
from CeibalSCAErrors import *
from CeibalSCAConf import *

from inspect import currentframe

class CeibalSCA_NM:
    def __init__(self, connect_callback, disconnect_callback):
        """ Create an instance of CeibalSCA_NM class.

            Arguments:
              + Callback function to call when connected to a Ceibal
                network. Must receive no arguments.
              + Callback function to call when disconnected of a Ceibal
                network. Must receive no arguments.
        """
        self.log = CeibalSCALog.get_instance()
        self.ceibal_conf = CeibalSCAConf.get_instance()
        self.ceibal_conf.add_config_changed_callback(self._config_changed)

        network = self.ceibal_conf.get_network_range()
        self._compute_network_range(network)

        self.connect_callback = connect_callback
        self.disconnect_callback = disconnect_callback

    def check_correct_network(self):
        """ Checks if we are connected to the correct network.
        """
        try:
            params = self._get_nm_connection_params()
        except Exception, e:
            self.log.debug("%s" % str(e))
            return False

        if (not params):
            return False

        if_name = params['if_name']
        network_name = params['network_name']
        ip_addr_int = params['ip_addr']

        ip_addr = join(map(str,unpack('4B', pack('<L', long(ip_addr_int)))), '.')
        ip_addr_int = self._ip2int(ip_addr)
        ip_addr_int = ip_addr_int & self.unsigned_mask

        if (not re.match("^%s$" % self.ceibal_conf.get_network_interface(), if_name)):
            # Incorrect network interface
            return False

        if (not re.match("^%s$" % self.ceibal_conf.get_network_essid(), network_name)):
            # Incorrect network essid
            return False

        if ((ip_addr_int < self.first_ip) or
            (ip_addr_int > self.last_ip)):
            # Incorrect IP range
            return False

        return True


    # PRIVATE FUNCTIONS
    # =================
    def _get_nm_connection_params(self):
        """ This function must return a dictionary with the following data:
              "if_name":      The interface name used in the NetworkManager connection
              "network_name": The SSID of the connected network.
              "ip_addr":      The assigned IP address, as returned by NM (integer).

            Subclasses must override this method to obtain and return
            this information from NetworkManager.

            If the interface is not connected, result should be None.
        """
        raise CeibalSCAException("Method not implemented. Please override _get_nm_connection_params method.")

    def _nm_connected (self):
        """ NetworkManager connected to a network. This function
            checks that it is the right network, and calls
            connect_callback.
        """
        if (not self.check_correct_network()):
            # Not in the correct network, we ignore it
            return

        # Message is not as informative, but we don't want to obtain
        # connection params again.
        self.log.debug("Connected to configured network.")

        if (self.connect_callback):
            # We must sleep, to allow DHCP client to obtain data
            time.sleep(3)
            self.connect_callback()

    def _nm_disconnected (self):
        if (self.disconnect_callback):
            self.disconnect_callback()


    def _config_changed(self):
        network = self.ceibal_conf.get_network_range()
        self._compute_network_range(network)

    def _compute_network_range(self, network):
        """ Computes the network range, and sets first_ip, last_ip and
            unsigned_mask attributes accordingly.

            Arguments:
              + Network range in <ip_addr>/<mask> format.
        """
        self.unsigned_mask = self._get_unsigned_mask()
        int_ip = self._ip2int(network)

        mask = self._get_mask(network)
        mask = mask & self.unsigned_mask

        self.first_ip = (int_ip & mask) + 1
        self.first_ip = self.first_ip & self.unsigned_mask
        self.last_ip  = (int_ip | ~mask) - 1
        self.last_ip = self.last_ip & self.unsigned_mask

    def _get_unsigned_mask(self):
        """ Returns the mask to convert a signed int to an unsigned
            one.
        """
        return long("0x" + ("F"*8), 16)

    def _int2ip(self, ip):
        """ Returns the dotted notation representation of a given IP
            address.

            Arguments:
              + The ip address as an integer
        """
        return join(map(str,unpack('4B', pack('>L', long(ip)))), '.')

    def _ip2int(self, ip):
        """ Returns the integer representation of a given IP address
            in dotted notation.

            Arguments:
              + The ip address in dotted notation
        """
        match = re.match("^(\d*)\.(\d*)\.(\d*)\.(\d*)", ip)
        if not match : return long(0)
        ip_list = list(match.groups())
        hexn = ''.join(["%02X" % long(i) for i in ip_list])
        return long(hexn, 16)


    def _get_mask(self, range):
        """ Returns the mask of the given IP range, as an integer.

            Arguments:
              + The IP range, in CIDR notation (ip_address/mask)
        """
        match = re.match("^\d*\.\d*\.\d*\.\d*/(\d+)", range)
        if not match: return 0
        one_bits = int(match.group(1))
        mask = 0
        for i in xrange(0, one_bits): mask = (mask << 1) + 1
        mask_dotted = join(map(str,unpack('4B', pack('<L', long(mask)))), '.')
        return self._ip2int(mask_dotted)

